@@ -249,9 +249,11 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->max_jobs           = GM_DEFAULT_MAX_JOBS;
     opt->spawn_rate         = GM_DEFAULT_SPAWN_RATE;
     opt->identifier         = NULL;
+    opt->queue_cust_var     = NULL;
     opt->show_error_output  = GM_ENABLED;
-    opt->orphan_host_checks    = GM_ENABLED;
-    opt->orphan_service_checks = GM_ENABLED;
+    opt->dup_results_are_passive = GM_ENABLED;
+    opt->orphan_host_checks      = GM_ENABLED;
+    opt->orphan_service_checks   = GM_ENABLED;
 
     opt->workaround_rc_25   = GM_DISABLED;
 
@@ -324,6 +326,7 @@ int parse_yes_or_no(char*value, int dfl) {
 
 /* parse one line of args into the given struct */
 int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
+    int x;
     char *key;
     char *value;
 
@@ -415,6 +418,12 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         return(GM_OK);
     }
 
+    /* dup_results_are_passive */
+    else if ( !strcmp( key, "dup_results_are_passive" ) ) {
+        opt->dup_results_are_passive = parse_yes_or_no(value, GM_ENABLED);
+        return(GM_OK);
+    }
+
     /* workaround_rc_25 */
     else if ( !strcmp( key, "workaround_rc_25" ) ) {
         opt->workaround_rc_25 = parse_yes_or_no(value, GM_ENABLED);
@@ -615,13 +624,13 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     /* max-age */
     else if ( !strcmp( key, "max-age" ) ) {
         opt->max_age = atoi( value );
-        if(opt->max_age <= 0) { opt->max_age = 1; }
+        if(opt->max_age < 0) { opt->max_age = GM_DEFAULT_JOB_MAX_AGE; }
     }
 
     /* idle-timeout */
     else if ( !strcmp( key, "idle-timeout" ) ) {
         opt->idle_timeout = atoi( value );
-        if(opt->idle_timeout < 0) { opt->job_timeout = GM_DEFAULT_IDLE_TIMEOUT; }
+        if(opt->idle_timeout < 0) { opt->idle_timeout = GM_DEFAULT_IDLE_TIMEOUT; }
     }
 
     /* max-jobs */
@@ -649,11 +658,7 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     else if ( !strcmp( key, "server" ) ) {
         char *servername;
         while ( (servername = strsep( &value, "," )) != NULL ) {
-            char * new_server = get_param_server(servername, opt->server_list, opt->server_num);
-            if(new_server != NULL) {
-                opt->server_list[opt->server_num] = new_server;
-                opt->server_num++;
-            }
+            add_server(&opt->server_num, opt->server_list, servername);
         }
     }
 
@@ -661,11 +666,7 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     else if ( !strcmp( key, "dupserver" ) ) {
         char *servername;
         while ( (servername = strsep( &value, "," )) != NULL ) {
-            char * new_server = get_param_server(servername, opt->dupserver_list, opt->dupserver_num);
-            if(new_server != NULL) {
-                opt->dupserver_list[opt->dupserver_num] = new_server;
-                opt->dupserver_num++;
-            }
+            add_server(&opt->dupserver_num, opt->dupserver_list, servername);
         }
     }
 
@@ -733,6 +734,15 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         }
     }
 
+    /* queue_custom_variable */
+    else if ( !strcmp( key, "queue_custom_variable" ) ) {
+        /* uppercase custom variable name */
+        for(x = 0; value[x] != '\x0'; x++) {
+            value[x] = toupper(value[x]);
+        }
+        opt->queue_cust_var = strdup( value );
+    }
+
     /* export queues */
     else if ( !strcmp( key, "export" ) ) {
         char *callback;
@@ -880,6 +890,7 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
 #endif
     }
     if(mode == GM_NEB_MODE) {
+        gm_log( GM_LOG_DEBUG, "queue by cust var:   %s\n", opt->queue_cust_var == NULL ? "no" : opt->queue_cust_var);
         gm_log( GM_LOG_DEBUG, "debug result:        %s\n", opt->debug_result == GM_ENABLED ? "yes" : "no");
         gm_log( GM_LOG_DEBUG, "result_worker:       %d\n", opt->result_workers);
         gm_log( GM_LOG_DEBUG, "do_hostchecks:       %s\n", opt->do_hostchecks == GM_ENABLED ? "yes" : "no");
@@ -891,10 +902,10 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
 
     /* server && queues */
     for(i=0;i<opt->server_num;i++)
-        gm_log( GM_LOG_DEBUG, "server:              %s\n", opt->server_list[i]);
+        gm_log( GM_LOG_DEBUG, "server:              %s:%i\n", opt->server_list[i]->host, opt->server_list[i]->port);
     gm_log( GM_LOG_DEBUG, "\n" );
     for(i=0;i<opt->dupserver_num;i++)
-        gm_log( GM_LOG_DEBUG, "dupserver:           %s\n", opt->dupserver_list[i]);
+        gm_log( GM_LOG_DEBUG, "dupserver:           %s:%i\n", opt->dupserver_list[i]->host, opt->dupserver_list[i]->port);
     gm_log( GM_LOG_DEBUG, "\n" );
     if(mode == GM_NEB_MODE) {
         gm_log( GM_LOG_DEBUG, "perfdata:            %s\n", opt->perfdata      == GM_ENABLED ? "yes" : "no");
@@ -947,10 +958,14 @@ void mod_gm_free_opt(mod_gm_opt_t *opt) {
     if(opt == NULL)
         return;
     int i,j;
-    for(i=0;i<opt->server_num;i++)
+    for(i=0;i<opt->server_num;i++) {
+        free(opt->server_list[i]->host);
         free(opt->server_list[i]);
-    for(i=0;i<opt->dupserver_num;i++)
+    }
+    for(i=0;i<opt->dupserver_num;i++) {
+        free(opt->dupserver_list[i]->host);
         free(opt->dupserver_list[i]);
+    }
     for(i=0;i<opt->hostgroups_num;i++)
         free(opt->hostgroups_list[i]);
     for(i=0;i<opt->servicegroups_num;i++)
@@ -973,6 +988,7 @@ void mod_gm_free_opt(mod_gm_opt_t *opt) {
     free(opt->host);
     free(opt->service);
     free(opt->identifier);
+    free(opt->queue_cust_var);
 #ifdef EMBEDDEDPERL
     free(opt->p1_file);
 #endif
@@ -1565,36 +1581,16 @@ void gm_log( int lvl, const char *text, ... ) {
     return;
 }
 
-/* extract server from string and check for duplicates */
-char * get_param_server(char * servername, char * server_list[GM_LISTSIZE], int server_num) {
-    char temp_buffer[GM_BUFFERSIZE];
-    char * new_server;
+/* check server for duplicates */
+int check_param_server(gm_server_t * new_server, gm_server_t * server_list[GM_LISTSIZE], int server_num) {
     int i;
-    servername = trim(servername);
-
-    if ( ! strcmp( servername, "" ) ) {
-        return NULL;
-    }
-
-    if(strcspn(servername, ":") == 0) {
-        temp_buffer[0]='\x0';
-        snprintf( temp_buffer,sizeof( temp_buffer )-1, "localhost%s", servername);
-        temp_buffer[sizeof( temp_buffer )-1]='\x0';
-        new_server = strdup(temp_buffer);
-    } else {
-        new_server = strdup(servername);
-    }
-
-    // check for duplicates
     for(i=0;i<server_num;i++) {
-        if ( ! strcmp( new_server, server_list[i] ) ) {
-            gm_log( GM_LOG_ERROR, "duplicate definition of server: %s\n", new_server);
-            free(new_server);
-            return NULL;
+        if ( ! strcmp( new_server->host, server_list[i]->host ) && new_server->port == server_list[i]->port ) {
+            gm_log( GM_LOG_ERROR, "duplicate definition of server: %s:%i\n", new_server->host, new_server->port);
+            return GM_ERROR;
         }
     }
-
-    return new_server;
+    return GM_OK;
 }
 
 
@@ -1624,8 +1620,8 @@ void send_result_back(gm_job_t * exec_job) {
     temp_buffer1[0]='\x0';
     snprintf( temp_buffer1, sizeof( temp_buffer1 )-1, "host_name=%s\ncore_start_time=%i.%i\nstart_time=%i.%i\nfinish_time=%i.%i\nreturn_code=%i\nexited_ok=%i\n",
               exec_job->host_name,
-              ( int )exec_job->core_start_time.tv_sec,
-              ( int )exec_job->core_start_time.tv_usec,
+              ( int )exec_job->next_check.tv_sec,
+              ( int )exec_job->next_check.tv_usec,
               ( int )exec_job->start_time.tv_sec,
               ( int )exec_job->start_time.tv_usec,
               ( int )exec_job->finish_time.tv_sec,
@@ -1686,7 +1682,9 @@ void send_result_back(gm_job_t * exec_job) {
     }
 
     if( mod_gm_opt->dupserver_num ) {
-        strncpy(temp_buffer2, "type=passive\n", (sizeof(temp_buffer1)-2));
+        if(mod_gm_opt->dup_results_are_passive) {
+            strncpy(temp_buffer2, "type=passive\n", (sizeof(temp_buffer1)-2));
+        }
         strncat(temp_buffer2, temp_buffer1, (sizeof(temp_buffer2)-2));
         temp_buffer2[sizeof( temp_buffer2 )-1]='\x0';
         if( add_job_to_queue( current_client_dup,
@@ -1728,3 +1726,32 @@ char *md5sum(char *text) {
 
     return result;
 }
+
+/* add parsed server to list */
+void add_server(int * server_num, gm_server_t * server_list[GM_LISTSIZE], char * servername) {
+    char * server   = strdup( servername );
+    char * server_c = server;
+    char * host     = strsep( &server, ":" );
+    char * port_val = strsep( &server, "\x0" );
+    in_port_t port  = GM_SERVER_DEFAULT_PORT;
+    if(port_val != NULL) {
+        port  = ( in_port_t ) atoi( port_val );
+    }
+    gm_server_t *new_server;
+    new_server = malloc(sizeof(gm_server_t));
+    if(!strcmp(host, "")) {
+        new_server->host = strdup("localhost");
+    } else {
+        new_server->host = strdup(host);
+    }
+    new_server->port = port;
+    if(check_param_server(new_server, server_list, *server_num) == GM_OK) {
+        server_list[*server_num] = new_server;
+        *server_num = *server_num + 1;
+    } else {
+        free(new_server->host);
+        free(new_server);
+    }
+    free(server_c);
+    return;
+}
