@@ -391,15 +391,22 @@ static int handle_eventhandler( int event_type, void *data ) {
     nebstruct_event_handler_data * ds;
     host * hst    = NULL;
     service * svc = NULL;
+    struct timeval core_time;
+    gettimeofday(&core_time,NULL);
 
-    gm_log( GM_LOG_DEBUG, "got eventhandler event\n" );
     gm_log( GM_LOG_TRACE, "handle_eventhandler(%i, data)\n", event_type );
 
-    if ( event_type != NEBTYPE_EVENTHANDLER_START )
+    if ( event_type != NEBCALLBACK_EVENT_HANDLER_DATA )
         return NEB_OK;
 
     ds = ( nebstruct_event_handler_data * )data;
 
+    if ( ds->type != NEBTYPE_EVENTHANDLER_START ) {
+        gm_log( GM_LOG_TRACE, "skiped type %i, expecting: %i\n", ds->type, NEBTYPE_EVENTHANDLER_START );
+        return NEB_OK;
+    }
+
+    gm_log( GM_LOG_DEBUG, "got eventhandler event\n" );
     gm_log( GM_LOG_TRACE, "got eventhandler event: %s\n", ds->command_line );
 
     /* service event handler? */
@@ -432,7 +439,13 @@ static int handle_eventhandler( int event_type, void *data ) {
     }
 
     temp_buffer[0]='\x0';
-    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=eventhandler\ncommand_line=%s\n\n\n",ds->command_line );
+    snprintf( temp_buffer,GM_BUFFERSIZE-1,
+                "type=eventhandler\nstart_time=%i.0\ncore_time=%i.%i\ncommand_line=%s\n\n\n",
+                (int)core_time.tv_sec,
+                (int)core_time.tv_sec,
+                (int)core_time.tv_usec,
+                ds->command_line
+    );
 
     if(add_job_to_queue( &client,
                          mod_gm_opt->server_list,
@@ -463,6 +476,8 @@ static int handle_host_check( int event_type, void *data ) {
     host * hst;
     check_result * chk_result;
     int check_options;
+    struct timeval core_time;
+    gettimeofday(&core_time,NULL);
 
     gm_log( GM_LOG_TRACE, "handle_host_check(%i)\n", event_type );
 
@@ -561,12 +576,11 @@ static int handle_host_check( int event_type, void *data ) {
 
     /* log latency */
     if(mod_gm_opt->debug_level >= GM_LOG_DEBUG) {
-        time_t t = time(NULL);
         struct tm next_check;
         localtime_r(&hst->next_check, &next_check);
         char buffer1[GM_BUFFERSIZE];
         strftime(buffer1, sizeof(buffer1), "%Y-%m-%d %H:%M:%S", &next_check );
-        gm_log( GM_LOG_DEBUG, "host: '%s', next_check is at %s, latency: %i\n", hst->name, buffer1, ((int) t - (int)hst->next_check));
+        gm_log( GM_LOG_DEBUG, "host: '%s', next_check is at %s, latency so far: %i\n", hst->name, buffer1, ((int)core_time.tv_sec - (int)hst->next_check));
     }
 
     /* increment number of host checks that are currently running */
@@ -577,11 +591,14 @@ static int handle_host_check( int event_type, void *data ) {
 
     gm_log( GM_LOG_TRACE, "cmd_line: %s\n", processed_command );
 
-    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=host\nresult_queue=%s\nhost_name=%s\nstart_time=%i.0\ntimeout=%d\ncommand_line=%s\n\n\n",
+    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=host\nresult_queue=%s\nhost_name=%s\nstart_time=%i.0\nnext_check=%i.0\ntimeout=%d\ncore_time=%i.%i\ncommand_line=%s\n\n\n",
               mod_gm_opt->result_queue,
               hst->name,
-              ( int )hst->next_check,
+              (int)hst->next_check,
+              (int)hst->next_check,
               host_check_timeout,
+              (int)core_time.tv_sec,
+              (int)core_time.tv_usec,
               processed_command
             );
 
@@ -629,6 +646,8 @@ static int handle_svc_check( int event_type, void *data ) {
     nebstruct_service_check_data * svcdata;
     int prio = GM_JOB_PRIO_LOW;
     check_result * chk_result;
+    struct timeval core_time;
+    gettimeofday(&core_time,NULL);
 
     gm_log( GM_LOG_TRACE, "handle_svc_check(%i, data)\n", event_type );
     svcdata = ( nebstruct_service_check_data * )data;
@@ -725,12 +744,11 @@ static int handle_svc_check( int event_type, void *data ) {
 
     /* log latency */
     if(mod_gm_opt->debug_level >= GM_LOG_DEBUG) {
-        time_t t = time(NULL);
         struct tm next_check;
         localtime_r(&svc->next_check, &next_check);
         char buffer1[GM_BUFFERSIZE];
         strftime(buffer1, sizeof(buffer1), "%Y-%m-%d %H:%M:%S", &next_check );
-        gm_log( GM_LOG_DEBUG, "service: '%s' - '%s', next_check is at %s, latency: %i\n", svcdata->host_name, svcdata->service_description, buffer1, ((int) t - (int)svc->next_check));
+        gm_log( GM_LOG_DEBUG, "service: '%s' - '%s', next_check is at %s, latency so far: %i\n", svcdata->host_name, svcdata->service_description, buffer1, ((int)core_time.tv_sec - (int)svc->next_check));
     }
 
     /* increment number of service checks that are currently running... */
@@ -741,11 +759,14 @@ static int handle_svc_check( int event_type, void *data ) {
 
     gm_log( GM_LOG_TRACE, "cmd_line: %s\n", processed_command );
 
-    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=service\nresult_queue=%s\nhost_name=%s\nservice_description=%s\nstart_time=%i.0\ntimeout=%d\ncommand_line=%s\n\n\n",
+    snprintf( temp_buffer,GM_BUFFERSIZE-1,"type=service\nresult_queue=%s\nhost_name=%s\nservice_description=%s\nstart_time=%i.0\nnext_check=%i.0\ncore_time=%i.%i\ntimeout=%d\ncommand_line=%s\n\n\n",
               mod_gm_opt->result_queue,
               svcdata->host_name,
               svcdata->service_description,
-              ( int )svc->next_check,
+              (int)svc->next_check,
+              (int)svc->next_check,
+              (int)core_time.tv_sec,
+              (int)core_time.tv_usec,
               service_check_timeout,
               processed_command
             );
@@ -868,10 +889,43 @@ static int verify_options(mod_gm_opt_t *opt) {
 /* return the prefered target function for our worker */
 static void set_target_queue( host *hst, service *svc ) {
     int x=0;
+    customvariablesmember *temp_customvariablesmember = NULL;
 
     /* empty our target */
     target_queue[0] = '\x0';
 
+    /* grab target queue from custom variable */
+    if( mod_gm_opt->queue_cust_var ) {
+        if( svc ) {
+            temp_customvariablesmember = svc->custom_variables;
+            for(; temp_customvariablesmember != NULL; temp_customvariablesmember = temp_customvariablesmember->next) {
+                if(!strcmp(mod_gm_opt->queue_cust_var, temp_customvariablesmember->variable_name)) {
+                    if(!strcmp(temp_customvariablesmember->variable_value, "local")) {
+                        gm_log( GM_LOG_TRACE, "bypassing local check from service custom variable\n" );
+                        return;
+                    }
+                    gm_log( GM_LOG_TRACE, "got target queue from service custom variable: %s\n", temp_customvariablesmember->variable_value );
+                    snprintf( target_queue, GM_BUFFERSIZE-1, "%s", temp_customvariablesmember->variable_value );
+                    return;
+                }
+            }
+        }
+
+        /* search in host custom variables */
+        temp_customvariablesmember = hst->custom_variables;
+        for(; temp_customvariablesmember != NULL; temp_customvariablesmember = temp_customvariablesmember->next) {
+            if(!strcmp(mod_gm_opt->queue_cust_var, temp_customvariablesmember->variable_name)) {
+                if(!strcmp(temp_customvariablesmember->variable_value, "local")) {
+                    gm_log( GM_LOG_TRACE, "bypassing local check from host custom variable\n" );
+                    return;
+                }
+                gm_log( GM_LOG_TRACE, "got target queue from host custom variable: %s\n", temp_customvariablesmember->variable_value );
+                snprintf( target_queue, GM_BUFFERSIZE-1, "%s", temp_customvariablesmember->variable_value );
+                return;
+            }
+        }
+    }
+
     /* look for matching local servicegroups */
     if ( svc ) {
         while ( mod_gm_opt->local_servicegroups_list[x] != NULL ) {
